#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_TELL_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_TELL_HPP

#include "pythonic/include/io/_io/TextIOWrapper/tell.hpp"
#include "pythonic/builtins/file/tell.hpp"

#endif
