#ifndef PYTHONIC_INCLUDE_NUMPY_FLOOR_DIVIDE_ACCUMULATE_HPP
#define PYTHONIC_INCLUDE_NUMPY_FLOOR_DIVIDE_ACCUMULATE_HPP

#define UFUNC_NAME floor_divide
#include "pythonic/include/numpy/ufunc_accumulate.hpp"

#endif
