#ifndef PYTHONIC_INCLUDE_OPERATOR_XOR__HPP
#define PYTHONIC_INCLUDE_OPERATOR_XOR__HPP

#include "pythonic/include/operator_/xor_.hpp"
#include "pythonic/include/utils/functor.hpp"

PYTHONIC_NS_BEGIN
namespace operator_
{
  USING_FUNCTOR(__xor__, xor_);
}
PYTHONIC_NS_END

#endif
