#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_ISATTY_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_ISATTY_HPP

#include "pythonic/include/io/_io/TextIOWrapper/isatty.hpp"
#include "pythonic/builtins/file/isatty.hpp"
#endif
