#ifndef PYTHONIC_INCLUDE_NUMPY_PRODUCT_HPP
#define PYTHONIC_INCLUDE_NUMPY_PRODUCT_HPP

#include "pythonic/include/numpy/prod.hpp"

PYTHONIC_NS_BEGIN
namespace numpy
{
  USING_FUNCTOR(product, prod);
}
PYTHONIC_NS_END

#endif
